'From Squeak3.6 of ''6 October 2003'' [latest update: #5429] on 29 June 2004 at 10:54:02 pm'!
"Change Set:		RikHax
Date:			24 February 2004
Author:			Rik Smoody

Various tool hacks that I like to have in an image.
'Move-in' code.
Copy them in good health."!


!Object methodsFor: 'error handling'!
shiftHalt
	"This is the typical message to use for inserting breakpoints during 
	debugging. It behaves like halt:, but does not call on halt: in order to 
	avoid putting this message on the stack. Halt is especially useful when 
	the breakpoint message is an arbitrary one."

	Sensor shiftPressed ifTrue: [
	   Debugger
		openContext: thisContext
		label: 'shiftHalt encountered.'
		contents: thisContext shortStack]

	"nil shiftHalt."! !

!Object methodsFor: 'printing' stamp: 'Rik 3/3/2004 00:27'!
printString
	"Answer a String whose characters are a description of the receiver. 
	If you want to print without a character limit, use fullPrintString."

	^ self printStringLimitedTo: 499999! !

!Object methodsFor: 'RikHax' stamp: 'Rik 2/2/2004 10:45'!
gomi
	self halt: 'invoki8ng method is apparently still being used'! !


!ChangeSet methodsFor: 'fileIn/Out' stamp: 'Rik 5/21/2004 14:18'!
fileOut
	"File out the receiver, to a file whose name is a function of the change-set name and either of the date & time or chosen to have a unique numeric tag, depending on the preference 'changeSetVersionNumbers'"

	| file slips nameToUse usedName |
	self checkForConversionMethods.
	nameToUse _ Preferences changeSetVersionNumbers
		ifTrue:
			[Preferences sourceCodeDirectory nextNameFor: self name extension: 'cs']
		ifFalse:
			[(self name, FileDirectory dot, Utilities dateTimeSuffix, 
				FileDirectory dot, 'cs') asFileName].

	(Preferences warningForMacOSFileNameLength and: [nameToUse size > 31])
		ifTrue: [
			nameToUse _ FillInTheBlank request:  (nameToUse , '\has ' , nameToUse size asString , ' letters - too long for Mac OS.\Suggested replacement is:') withCRs initialAnswer: (nameToUse contractTo:30).
			nameToUse = '' ifTrue:[^self]].

	Cursor write showWhile:
		[[file _ Preferences sourceCodeDirectory newFileNamed: nameToUse.
		usedName _ file fullName.
		file header; timeStamp.
		self fileOutPreambleOn: file.
		self fileOutOn: file.
		self fileOutPostscriptOn: file.
		file trailer] ensure: [file close]].
	Transcript show: 'ChangeSorter fileIntoNewChangeSet: '''; show: usedName; show: '''.'; cr.
	Smalltalk logChange: 'ChangeSorter fileIntoNewChangeSet: ''', usedName, ''''.

	Preferences checkForSlips ifFalse: [^ self].

	slips _ self checkForSlips.
	(slips size > 0 and: [(PopUpMenu withCaption: 'Methods in this fileOut have halts
or references to the Transcript
or other ''slips'' in them.
Would you like to browse them?' chooseFrom: 'Ignore\Browse slips') = 2])
		ifTrue: [self systemNavigation  browseMessageList: slips
							name: 'Possible slips in ', name]
! !


!ChangeSetCategory methodsFor: 'services' stamp: 'Rik 4/5/2004 14:26'!
fileOutAllChangeSetsTo: aChangeSet
	"File out all the nonempty change sets in the current category, suppressing the checks for slips that might otherwise ensue.  Obtain user confirmation before undertaking this possibly prodigious task."

	| aList |
	aList _ self elementsInOrder.
	aList _ aList copyStartingWith: aChangeSet.
	aList _ aList select: [:ecs  | ecs isEmpty not].
	
	aList size == 0 ifTrue: [^ self inform: 'sorry, all the change sets in this category are empty'].
	(self confirm: 'This will result in filing out ', aList size printString, ' change set(s)
Are you certain you want to do this?') ifFalse: [^ self].

	Preferences setFlag: #checkForSlips toValue: false during: 
		[Utilities fileOutChangeSetsNamed: (aList collect: [:m | m name])]! !


!DualChangeSorter methodsFor: 'initialization' stamp: 'Rik 5/21/2004 12:24'!
initialExtent
	"Answer the desired extent for the receiver when it is first opened on the screen.  "

	^ 550 @ ((Display extent y * 0.9) rounded min: 1200)! !


!ParagraphEditor methodsFor: 'menu messages' stamp: 'Rik 3/29/2004 16:17'!
find
	"Prompt the user for a string to search for, and search the receiver from the current selection onward for it.  1/26/96 sw"

	| reply |
	reply _ FillInTheBlank request: 'Find what? ' initialAnswer: Clipboard clipboardText.
	reply size == 0 ifTrue: [^ self].
	self setSearch: reply.
	ChangeText _ FindText.  "Implies no replacement to againOnce: method"
	self againOrSame: true
	
! !


!PositionableStream methodsFor: 'fileIn/Out' stamp: 'Rik 2/24/2004 20:36'!
checkForPreamble: chunk
	| lStream |
	(chunk beginsWith: '"Change Set:') ifTrue: [
		lStream _ chunk readStream nextLine readStream.
		lStream upTo: $:; skipSeparators.
		ChangeSet  newChanges: (ChangeSorter assuredChangeSetNamed: lStream upToEnd).
		ChangeSet current preambleString: chunk].

	((chunk beginsWith: '"Postscript:') and: [ChangeSet current postscriptString == nil])
		ifTrue: [ChangeSet current postscriptString: chunk].
							
! !


!Preferences class methodsFor: 'hard-coded prefs' stamp: 'Rik 2/2/2004 12:25'!
browseToolClass
	^ Preferences browserShowsPackagePane
		ifTrue:
			[PackagePaneBrowser]
		ifFalse:
			[HierarchyBrowser]! !

!Preferences class methodsFor: 'hard-coded prefs' stamp: 'Rik 5/21/2004 13:44'!
sourceCodeDirectory
	"Preferences sourceCodeDirectory"

	(FileDirectory default directoryExists: 'cs')
		ifFalse: [FileDirectory default createDirectory: 'cs'].
	^FileDirectory default directoryNamed: 'cs'! !



!SequenceableCollection methodsFor: 'accessing' stamp: 'Rik 2/24/2004 20:40'!
after: target ifAbsent: exceptionBlock
	"Answer the element after target.  Answer the result of evaluation
	the exceptionBlock if target is not in the receiver, or if there are 
	no elements after it."

	| index |
	index _ self indexOf: target.
	^ index == 0
		ifTrue: [exceptionBlock value]
		ifFalse: [index = self size 
			ifTrue: [exceptionBlock value]
			ifFalse: [self at: index + 1]]! !

!SequenceableCollection methodsFor: 'copying' stamp: 'Rik 4/5/2004 12:05'!
copyStartingWith: anElement
	"Answer a copy of the receiver from the first occurence of anElement up to the end.
	If no such element exists, answer an empty copy."

	^ self allButFirst: (self indexOf: anElement ifAbsent: [^ self copyEmpty]) - 1

"
#(a b c d e f g ) copyStartingWith: #c #(#c #d #e #f #g)
#(a b c d e f g ) copyStartingWith: #a #(#a #b #c #d #e #f #g)
#(a b c d e f g ) copyStartingWith: #x #()
"! !

!SequenceableCollection methodsFor: 'copying' stamp: 'Rik 4/5/2004 12:06'!
copyUpThrough: anElement 
	"Answer all elements up to and including anObject. If there
	is no such object, answer a copy of the receiver."

	^ self first: (self indexOf: anElement ifAbsent: [^ self copy])


"
#(a b c d e f g ) copyUpThrough: #c  #(#a #b #c)
#(a b c d e f g ) copyUpThrough: #a  #(#a)
#(a b c d e f g ) copyUpThrough: #g   #(#a #b #c #d #e #f #g)
#(a b c d e f g ) copyUpThrough: #x  #(#a #b #c #d #e #f #g)
"! !


!Heap methodsFor: 'comparing' stamp: 'Rik 2/4/2004 12:28'!
= aHeap
	"Answer true if my and aHeap's classes are the same,
	and if our blocks are the same, and if our elements are the same.
	Future consideration: a collection of another species with all the same elements could be considered =, but perhqaps it's best not to overload the semantics of =  -Rik"

	self class = aHeap class ifFalse: [^ false].
	sortBlock = aHeap sortBlock
		ifTrue: [^ super = aHeap]
		ifFalse: [^ false]! !


!Interval methodsFor: 'comparing' stamp: 'Rik 2/4/2004 12:21'!
= other 
	"Answer true if we are of same class
	if our starts, steps and sizes are equal.
 	OR if the other is a collection of my same elements"

	other isCollection ifFalse: [^false].
	self size = other size  ifFalse: [^false].
	self class == other class
		ifTrue: [
				^start = other first
					and: [step = other increment]]
		ifFalse: [
				self with: other do: [ :mine :his | mine = his  ifFalse: [^false] ].
				^true]! !


!SmallInteger methodsFor: 'printing' stamp: 'Rik 5/23/2004 15:35'!
printDigitsOn: aStream base: b
	"just the digits, not the radix"

	| digitsInReverse x i |
	self < 0 ifTrue: [
		aStream nextPut: $-.
		^ self negated printOn: aStream base: b.
	].

	digitsInReverse _ Array new: 32.
	x _ self.
	i _ 0.
	[x >= b] whileTrue: [
		digitsInReverse at: (i _ i + 1) put: x \\ b.
		x _ x // b.
	].
	digitsInReverse at: (i _ i + 1) put: x.
	[i > 0] whileTrue: [
		aStream nextPut: (Character digitValue: (digitsInReverse at: i)).
		i _ i - 1.
	].! !

!SmallInteger methodsFor: 'printing' stamp: 'Rik 5/23/2004 21:52'!
printMultiplicationTableOn: aStream 
	aStream nextPutAll: '!!Base '; print: self; 
		cr; nextPutAll: '| &star; '.
	1 to: self - 1 do: [:xx | 
		aStream nextPutAll: ' | '.
		xx printDigitsOn: aStream base: self].
	1 to: self - 1 do: [:yy | 
		aStream cr; nextPutAll: '| '.
		yy printDigitsOn: aStream base: self.
		1 to: self -1 do: [:xx | 
			aStream nextPutAll: ' | '.
			xx < yy ifTrue: [aStream nextPutAll: '<font color="red">' ].
			xx > yy ifTrue: [aStream nextPutAll: '<font color="green">' ].
			xx * yy printDigitsOn: aStream base: self.
			xx == yy ifFalse: [aStream nextPutAll: '</font> ' ].
			].
		].
	aStream cr; cr! !


!StringHolder methodsFor: 'initialize-release' stamp: 'Rik 5/21/2004 12:25'!
initialExtent
	"Answer the desired extent for the receiver when it is first opened on the screen.  "

	^ 550 @ ((Display extent y * 0.9) rounded min: 1200)! !


!Browser methodsFor: 'initialize-release' stamp: 'Rik 3/7/2004 08:01'!
defaultBrowserTitle
	^ 'SysB'! !

!Browser methodsFor: 'message category functions' stamp: 'Rik 3/7/2004 07:58'!
buildMessageCategoryBrowserEditString: aString 
	"Create and schedule a message category browser for the currently 
	selected	 message category. The initial text view contains the characters 
	in aString."
	"wod 6/24/1998: set newBrowser classListIndex so that it works whether the
	receiver is a standard or a Hierarchy Browser."

	| newBrowser |
	messageCategoryListIndex ~= 0
		ifTrue: 
			[newBrowser _ self class new.
			newBrowser systemCategoryListIndex: systemCategoryListIndex.
			newBrowser classListIndex: (newBrowser classList indexOf: self selectedClassName).
			newBrowser metaClassIndicated: metaClassIndicated.
			newBrowser messageCategoryListIndex: messageCategoryListIndex.
			newBrowser messageListIndex: messageListIndex.
			self class openBrowserView: (newBrowser openMessageCatEditString: aString)
				label: 'Msg CatB(' , 
						newBrowser selectedClassOrMetaClassName , ')']! !

!Browser methodsFor: 'message category functions' stamp: 'Rik 2/19/2004 00:31'!
fileOutMessageCategories
	"Print a description of the selected message category of the selected class 
	onto an external file."

Cursor write showWhile:
	[messageCategoryListIndex ~= 0
		ifTrue: [
			self selectedClassOrMetaClass fileOutCategory: self selectedMessageCategoryName]
		ifFalse: [
			self halt]]! !

!Browser methodsFor: 'system category functions' stamp: 'Rik 2/3/2004 13:01'!
alphabetizeClassCategories
	self okToChange
		ifFalse: [^ false].
	systemOrganizer sortCategories.
	self changed: #systemCategoryList.
	^ true! !

!Browser methodsFor: 'system category functions' stamp: 'Rik 2/14/2004 18:28'!
alphabetizeSystemCategories
	| comeBackTo comeBackToClass |
	self okToChange ifFalse: [^ self].
	comeBackTo _ self selectedSystemCategoryName.
	comeBackToClass _ self selectedClass.

	Cursor execute showWhile: [
		systemOrganizer sortCategories.
		self systemCategoryListIndex: (self systemCategoryList indexOf: comeBackTo).
		self selectClass: comeBackToClass

	].
	^ true! !

!Browser methodsFor: 'system category functions' stamp: 'Rik 2/3/2004 13:04'!
systemCategoryMenu: aMenu 
	^ aMenu
		labels: 'find class... (f)
recent classes... (r)
browse all
browse
printOut
fileOut
alphabetize
reorganize
update
add item...
rename...
remove'
		lines: #(2 4 6 9 )
		selections: #(#findClass #recent #browseAllClasses #buildSystemCategoryBrowser #printOutSystemCategory #fileOutSystemCategory #alphabetizeClassCategories #editSystemCategories #updateSystemCategories #addSystemCategory #renameSystemCategory #removeSystemCategory )! !


!ChangeList methodsFor: 'initialization-release' stamp: 'Rik 2/24/2004 20:50'!
initialExtent
	"Answer the desired extent for the receiver when it is first opened on the screen.  "

	^ 550 @ ((Display extent y * 0.9) rounded min: 1200)! !

!ChangeList methodsFor: 'scanning' stamp: 'Rik 5/21/2004 12:19'!
scanFile: aFile from: startPosition to: stopPosition
	| itemPosition item prevChar |
	file _ aFile.
	changeList _ OrderedCollection new.
	list _ OrderedCollection new.
	listIndex _ 0.
	file position: startPosition.
'Scanning ', aFile localName, '...'
	displayProgressAt: Sensor cursorPoint
	from: startPosition to: stopPosition
	during: [:bar |
	[file position < stopPosition]
		whileTrue:
		[bar value: file position.
		[file atEnd not and: [file peek isSeparator]]
				whileTrue: [prevChar _ file next].
		(file peekFor: $!!)
		ifTrue:
			[(prevChar = Character cr or: [prevChar = Character lf])
				ifTrue: [self scanCategory]]
		ifFalse:
			[itemPosition _ file position.
			item _ file nextChunk.
			file skipStyleChunk.
			item size > 0 ifTrue:
				[self addItem: (ChangeRecord new file: file position: itemPosition type: #doIt)
					text: 'do it: ' , (item contractTo: 99)]]]].
	listSelections _ Array new: list size withAll: false! !


!ChangeSorter methodsFor: 'changeSet menu' stamp: 'Rik 3/31/2004 16:28'!
changeSetMenu: aMenu shifted: isShifted 
	"Set up aMenu to hold commands for the change-set-list pane.  This could be for a single or double changeSorter"

	isShifted ifTrue: [^ self shiftedChangeSetMenu: aMenu].
	Smalltalk isMorphic
		ifTrue:
			[aMenu title: 'Change Set'.
			aMenu addStayUpItemSpecial]
		ifFalse:
			[aMenu title: 'Change Set:
' , myChangeSet name].

	aMenu add: 'make changes go to me (m)' action: #newCurrent.
	aMenu addLine.
	aMenu add: 'new change set... (n)' action: #newSet.
	aMenu add: 'find...(f)' action: #findCngSet.
	aMenu add: 'show category... (s)' action:  #chooseChangeSetCategory.
	aMenu balloonTextForLastItem:
'Lets you choose which change sets should be listed in this change sorter'.
	aMenu add: 'select change set...' action: #chooseCngSet.
	aMenu addLine.
	aMenu add: 'rename change set (r)' action: #rename.
	aMenu add: 'file out (o)' action: #fileOut.
	aMenu add: 'file out all to here' action: #fileOutAllToHere.
	aMenu add: 'mail to list' action: #mailOut.
	aMenu add: 'browse methods (b)' action: #browseChangeSet.
	aMenu add: 'browse change set (B)' action: #openChangeSetBrowser.
	aMenu addLine.
	parent
		ifNotNil: 
			[aMenu add: 'copy all to other side (c)' action: #copyAllToOther.
			aMenu add: 'submerge into other side' action: #submergeIntoOtherSide.
			aMenu add: 'subtract other side (-)' action: #subtractOtherSide.
			aMenu addLine].
	myChangeSet hasPreamble
		ifTrue: 
			[aMenu add: 'edit preamble (p)' action: #addPreamble.
			aMenu add: 'remove preamble' action: #removePreamble]
		ifFalse: [aMenu add: 'add preamble (p)' action: #addPreamble].
	myChangeSet hasPostscript
		ifTrue: 
			[aMenu add: 'edit postscript...' action: #editPostscript.
			aMenu add: 'remove postscript' action: #removePostscript]
		ifFalse: [aMenu add: 'add postscript...' action: #editPostscript].
	aMenu addLine.

	aMenu add: 'category functions...' action: #offerCategorySubmenu.
	aMenu balloonTextForLastItem:
'Various commands relating to change-set-categories'.
	aMenu addLine.


	aMenu add: 'destroy change set (x)' action: #remove.
	aMenu addLine.
	aMenu add: 'more...' action: #offerShiftedChangeSetMenu.
	^ aMenu! !

!ChangeSorter methodsFor: 'changeSet menu' stamp: 'Rik 4/5/2004 11:50'!
fileOutAllToHere
	self changeSetCategory fileOutAllChangeSetsTo: myChangeSet! !

!ChangeSorter methodsFor: 'class list' stamp: 'Rik 2/22/2004 23:50'!
forgetClass
	"Remove all mention of this class from the changeSet"

	| nextClassName |
	self okToChange ifFalse: [^ self].
	currentClassName ifNotNil: [
		nextClassName _ myChangeSet changedClassNames after: currentClassName ifAbsent: [nil]
.		myChangeSet removeClassChanges: currentClassName.
		currentClassName _ nextClassName.
		currentSelector _ nil.
		self showChangeSet: myChangeSet.
		self currentClassName: nextClassName].
! !


!Debugger methodsFor: 'initialize' stamp: 'Rik 3/1/2004 10:24'!
initialExtent
	"Make the full debugger longer!!"

	dependents size < 9 ifTrue: [^ super initialExtent].	"Pre debug window"
	RealEstateAgent standardWindowExtent y < 400 "a tiny screen" 
		ifTrue: [^ super initialExtent].
	
	^ 600@(999 min: (Display extent y - 100))
! !


!FileContentsBrowser methodsFor: 'initialize-release' stamp: 'Rik 3/7/2004 08:01'!
defaultBrowserTitle
	^ 'FileB'! !


!FileList class methodsFor: 'file reader registration' stamp: 'nk 12/7/2002 12:53'!
itemsForFile: fullName
	"Answer a list of services appropriate for a file of the given full name"

	| services suffix |
	suffix _ self suffixOf: fullName.
	services _ OrderedCollection new.
	self registeredFileReaderClasses do: [:reader |
		reader ifNotNil: [services addAll: (reader fileReaderServicesForFile: fullName suffix: suffix)]].
	^ services! !


!FillInTheBlank class methodsFor: 'instance creation' stamp: 'Rik 3/29/2004 16:20'!
request: queryString 
	"Create an instance of me whose question is queryString. Invoke it 
	centered at the cursor, and answer the string the user accepts. Answer 
	the empty string if the user cancels."

	"FillInTheBlank request: 'Your name?'"

	^ self
		request: queryString
		initialAnswer: Clipboard clipboardText
		centerAt: (ActiveHand ifNil:[Sensor]) cursorPoint! !


!HierarchyBrowser methodsFor: 'initialization' stamp: 'Rik 3/7/2004 08:00'!
defaultBrowserTitle
	^ 'HB'! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'Rik 3/1/2004 10:32'!
initialExtent
	RealEstateAgent standardWindowExtent y < 400 "a tiny screen" 
		ifTrue: [^ super initialExtent].
	
	^ 600@(999 min: (Display extent y * 0.8) rounded)
! !

!HierarchyBrowser methodsFor: 'initialization' stamp: 'Rik 3/12/2004 11:13'!
setClass: aBehavior selector: aSymbol
	self initHierarchyForClass: aBehavior.
	"super setClass: aBehavior selector: aSymbol"! !


!Inspector methodsFor: 'accessing' stamp: 'Rik 2/18/2004 15:37'!
initialExtent
	"Answer the desired extent for the receiver when it is first opened on the screen.  "

	^ 250 @ 400! !

!Inspector methodsFor: 'selecting' stamp: 'Rik 5/25/2004 08:10'!
selectionPrintString
	| text nm |
	selectionUpdateTime _ [text _ [self selection printStringLimitedTo: 52000]
		on: Error do: 
		[nm _ self selectionIndex < 3
					ifTrue: ['self']
					ifFalse: [self selectedSlotName].
		text _ ('<error in printString: evaluate "' , nm , ' printString" to debug>') asText.
		text
			addAttribute: TextColor red
			from: 1
			to: text size.
		text]] timeToRun.
	^ text! !


!MessageSet methodsFor: 'message functions' stamp: 'Rik 2/19/2004 01:44'!
fileOutMessage
	| fileStream nix fname |
	self selectedMessageName isNil ifFalse: [
		^super fileOutMessage].
	"File out all messages"
	messageList isEmpty ifTrue: [^self].
	nix _ 0.
	[FileStream isAFileNamed: (fname _ 'MethodList', nix asString, '.st')] whileTrue: [nix _ nix+1].
	fileStream _ FileStream newFileNamed: fname.
	fileStream header; timeStamp.
	messageList do: [ :each |
		each actualClass  printMethodChunk: each methodSymbol withPreamble: true
				on: fileStream moveSource: false toFile: 0.
		].
	fileStream trailer; close



"self selectedClassOrMetaClass fileOutMethod: self selectedMessageName

prompt
"! !

!MessageSet methodsFor: 'initialize-release' stamp: 'Rik 3/8/2004 07:49'!
initialExtent
	RealEstateAgent standardWindowExtent y < 400 "a tiny screen" 
		ifTrue: [^ super initialExtent].
	
	^ 600@(999 min: (Display extent y * 0.8) rounded)
! !


!MessageSet class methodsFor: 'instance creation' stamp: 'Rik 2/19/2004 00:33'!
openAsMorph: aMessageSet name: labelString inWorld: aWorld
	^aMessageSet openAsMorphNamed: labelString inWorld: aWorld
! !


!PackagePaneBrowser methodsFor: 'initialize-release' stamp: 'Rik 3/7/2004 08:01'!
defaultBrowserTitle
	^ 'Pkg B'! !


!SelectorBrowser methodsFor: 'as yet unclassified' stamp: 'Rik 2/18/2004 15:41'!
initialExtent

	^ 350@450
! !


!TranscriptStream methodsFor: 'stream extensions' stamp: 'Rik 2/17/2004 01:21'!
crFlush
	self cr; endEntry! !

